from fastapi import APIRouter, Depends, HTTPException
from typing import Dict, Any, Optional, Union, List
from mcpstore.core.orchestrator import MCPOrchestrator
from mcpstore.core.registry import ServiceRegistry
from mcpstore.scripts.deps import get_orchestrator, get_registry
from pydantic import BaseModel, Field
import logging
import asyncio
from enum import Enum
from mcpstore.scripts.models import ToolExecutionRequest, ToolExecutionResponse, ContentUnion

router = APIRouter()
logger = logging.getLogger(__name__)

class ContentType(str, Enum):
    """内容类型枚举"""
    TEXT = "text"
    IMAGE = "image"
    JSON = "json"
    BINARY = "binary"

class Content(BaseModel):
    """基础内容模型"""
    type: ContentType
    mime_type: Optional[str] = None

class TextContent(Content):
    """文本内容"""
    type: ContentType = ContentType.TEXT
    text: str

class ImageContent(Content):
    """图像内容"""
    type: ContentType = ContentType.IMAGE
    url: str
    alt: Optional[str] = None

class JsonContent(Content):
    """JSON内容"""
    type: ContentType = ContentType.JSON
    data: Dict[str, Any]

class BinaryContent(Content):
    """二进制内容"""
    type: ContentType = ContentType.BINARY
    data: bytes
    filename: Optional[str] = None

ContentUnion = Union[TextContent, ImageContent, JsonContent, BinaryContent]

class ToolExecutionRequest(BaseModel):
    """工具执行请求模型"""
    tool_name: str
    parameters: Dict[str, Any]
    timeout: Optional[float] = Field(None, description="执行超时时间(秒)")

class ToolExecutionResponse(BaseModel):
    """工具执行响应模型"""
    result: Optional[List[ContentUnion]] = None
    status: str = "success"
    error: Optional[str] = None

@router.post("/execute", response_model=ToolExecutionResponse)
async def execute_tool(
    request: ToolExecutionRequest,
    orchestrator: MCPOrchestrator = Depends(get_orchestrator),
    registry: ServiceRegistry = Depends(get_registry)
):
    """执行工具
    
    Args:
        request: 工具执行请求
            - tool_name: 工具名称
            - parameters: 工具参数
            - timeout: 可选的超时时间(秒)
    """
    try:
        # 1. 验证工具是否存在
        tool_info = None
        for tool in registry.get_all_tool_info():
            if tool["name"] == request.tool_name:
                tool_info = tool
                break
                
        if not tool_info:
            raise HTTPException(
                status_code=404,
                detail=f"Tool not found: {request.tool_name}"
            )
            
        # 2. 获取服务信息
        service_name = tool_info["service_name"]
        if not service_name:
            raise HTTPException(
                status_code=500,
                detail=f"Service not found for tool: {request.tool_name}"
            )
            
        # 3. 验证服务健康状态
        is_healthy = await orchestrator.is_service_healthy(service_name)
        if not is_healthy:
            raise HTTPException(
                status_code=503,
                detail=f"Service {service_name} is unhealthy"
            )
            
        # 4. 验证参数
        input_schema = tool_info.get("inputSchema")
        if input_schema:
            # TODO: 实现参数验证
            pass
            
        # 5. 执行工具(带超时)
        try:
            if request.timeout:
                # 使用 asyncio.wait_for 实现超时
                result = await asyncio.wait_for(
                    orchestrator.execute_tool(
                        service_name=service_name,
                        tool_name=request.tool_name,
                        parameters=request.parameters
                    ),
                    timeout=request.timeout
                )
            else:
                result = await orchestrator.execute_tool(
                    service_name=service_name,
                    tool_name=request.tool_name,
                    parameters=request.parameters
                )
            
            # 处理结果，转换为适当的内容类型
            processed_results = []
            if isinstance(result, (list, tuple)):
                for item in result:
                    processed_results.extend(process_result_item(item))
            else:
                processed_results.extend(process_result_item(result))
            
            return ToolExecutionResponse(
                result=processed_results,
                status="success"
            )
            
        except asyncio.TimeoutError:
            logger.error(f"Tool execution timed out after {request.timeout} seconds")
            return ToolExecutionResponse(
                result=None,
                status="error",
                error=f"Execution timed out after {request.timeout} seconds"
            )
            
        except Exception as e:
            logger.error(f"Tool execution failed: {e}", exc_info=True)
            return ToolExecutionResponse(
                result=None,
                status="error",
                error=str(e)
            )
            
    except HTTPException:
        raise
    except Exception as e:
        logger.error(f"Unexpected error during tool execution: {e}", exc_info=True)
        raise HTTPException(
            status_code=500,
            detail=f"Internal server error: {str(e)}"
        )

def process_result_item(item: Any) -> List[ContentUnion]:
    """处理单个结果项，转换为适当的内容类型"""
    if isinstance(item, str):
        return [TextContent(text=item)]
    elif isinstance(item, bytes):
        return [BinaryContent(data=item)]
    elif isinstance(item, dict):
        return [JsonContent(data=item)]
    elif isinstance(item, (list, tuple)):
        results = []
        for sub_item in item:
            results.extend(process_result_item(sub_item))
        return results
    else:
        # 尝试转换为字符串
        return [TextContent(text=str(item))] 



import os, sys
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
import logging
from datetime import datetime
from typing import Dict, Any, Optional, Tuple, List, Set, TypeVar, Generic, Protocol

logger = logging.getLogger(__name__)

# 定义一个协议，表示任何具有call_tool方法的会话类型
class SessionProtocol(Protocol):
    async def call_tool(self, tool_name: str, tool_args: Dict[str, Any]) -> Any:
        ...

# 会话类型变量
SessionType = TypeVar('SessionType')

class ServiceRegistry:
    """Manages the state of connected services and their tools."""
    def __init__(self):
        self.sessions: Dict[str, Any] = {}  # service_name -> session
        self.service_health: Dict[str, datetime] = {} # service_name -> last_heartbeat_time
        self.tool_cache: Dict[str, Dict[str, Any]] = {} # tool_name -> tool_definition
        self.tool_to_session_map: Dict[str, Any] = {} # tool_name -> session
        logger.info("ServiceRegistry initialized.")

    def clear(self):
        """清空所有注册的服务和工具"""
        self.sessions.clear()
        self.service_health.clear()
        self.tool_cache.clear()
        self.tool_to_session_map.clear()

    def add_service(self, name: str, session: Any, tools: List[Tuple[str, Dict[str, Any]]]) -> List[str]:
        """Adds a new service, its session, and tools to the registry. Returns added tool names."""
        print(f"[DEBUG][add_service] name={name}, id(session)={id(session)}")
        if name in self.sessions:
            logger.warning(f"Attempting to add already registered service: {name}. Removing old service before overwriting.")
            self.remove_service(name)

        self.sessions[name] = session
        self.service_health[name] = datetime.now() # Mark healthy on add

        added_tool_names = []
        for tool_name, tool_definition in tools:
            # 检查工具名称是否属于当前服务
            if not tool_name.startswith(f"{name}_"):
                logger.warning(f"Tool '{tool_name}' does not belong to service '{name}'. Skipping this tool.")
                continue
                
            # 检查工具名称是否已存在
            if tool_name in self.tool_cache:
                # 如果工具已存在，检查是否属于同一个服务
                existing_session = self.tool_to_session_map.get(tool_name)
                if existing_session is not session:
                    logger.warning(f"Tool name conflict: '{tool_name}' from {name} conflicts with existing tool. Skipping this tool.")
                    continue
                    
            # 添加工具到缓存
            self.tool_cache[tool_name] = tool_definition
            self.tool_to_session_map[tool_name] = session
            added_tool_names.append(tool_name)
            
        logger.info(f"Service '{name}' added with tools: {added_tool_names}")
        return added_tool_names

    def remove_service(self, name: str) -> Optional[Any]:
        """Removes a service and its associated tools from the registry."""
        session = self.sessions.pop(name, None)
        
        if not session:
            logger.warning(f"Attempted to remove non-existent service: {name}")
            return None

        # Remove health record
        if name in self.service_health:
            del self.service_health[name]

        # Remove associated tools efficiently
        tools_to_remove = [tool_name for tool_name, owner_session in self.tool_to_session_map.items() if owner_session == session]
        if tools_to_remove:
            logger.info(f"Removing tools from registry associated with {name}: {tools_to_remove}")
            for tool_name in tools_to_remove:
                if tool_name in self.tool_cache: del self.tool_cache[tool_name]
                if tool_name in self.tool_to_session_map: del self.tool_to_session_map[tool_name]

        logger.info(f"Service '{name}' removed from registry.")
        return session

    def get_session(self, name: str) -> Optional[Any]:
        """Get session by service name"""
        return self.sessions.get(name)

    def get_session_for_tool(self, tool_name: str) -> Optional[Any]:
        """Get session by tool name"""
        return self.tool_to_session_map.get(tool_name)

    def get_all_tools(self) -> List[Dict[str, Any]]:
        """获取所有工具的定义"""
        all_tools = []
        
        # 遍历所有工具并添加服务信息
        for tool_name, tool_def in self.tool_cache.items():
            # 获取工具所属的服务信息
            session = self.tool_to_session_map.get(tool_name)
            service_name = None
            
            # 查找工具所属的服务
            for name, sess in self.sessions.items():
                if sess is session:
                    service_name = name
                    break
            
            # 创建包含服务信息的工具定义
            tool_with_service = tool_def.copy()
            
            # 如果是第一层级不包含function的情况，需要调整结构
            if "function" not in tool_with_service and isinstance(tool_with_service, dict):
                tool_with_service = {
                    "type": "function",
                    "function": tool_with_service
                }
            
            # 添加服务信息到函数名称或描述
            if "function" in tool_with_service:
                function_data = tool_with_service["function"]
                
                # 添加服务信息到描述中
                if service_name:
                    original_description = function_data.get("description", "")
                    if not original_description.endswith(f" (来自服务: {service_name})"):
                        function_data["description"] = f"{original_description} (来自服务: {service_name})"
                
                # 在内部保存服务信息，便于后续使用
                function_data["service_info"] = {
                    "service_name": service_name
                }
            
            all_tools.append(tool_with_service)
        
        logger.info(f"Returning {len(all_tools)} tools from {len(self.get_all_service_names())} services")
        return all_tools
        
    def get_all_tool_info(self) -> List[Dict[str, Any]]:
        """获取所有工具的详细信息"""
        tools_info = []
        for tool_name in self.tool_cache.keys():
            # 获取工具所属的服务
            session = self.tool_to_session_map.get(tool_name)
            service_name = None
            
            # 查找工具所属的服务名称
            for name, sess in self.sessions.items():
                if sess is session:
                    service_name = name
                    break
            
            # 获取详细工具信息
            detailed_tool = self._get_detailed_tool_info(tool_name)
            if detailed_tool:
                # 添加服务信息
                detailed_tool["service_name"] = service_name
                tools_info.append(detailed_tool)
            
        return tools_info
        
    def get_connected_services(self) -> List[Dict[str, Any]]:
        """获取所有已连接服务的信息"""
        services = []
        for name in self.get_all_service_names():
            tools = self.get_tools_for_service(name)
            services.append({
                "name": name,
                "tool_count": len(tools)
            })
        return services

    def get_tools_for_service(self, name: str) -> List[str]:
        """Get list of tools provided by the specified service"""
        session = self.sessions.get(name)
        logger.info(f"Getting tools for service: {name}")
        print(f"[DEBUG][get_tools_for_service] name={name}, id(session)={id(session) if session else None}")
        
        if not session:
            return []
            
        # 找到所有属于这个服务的工具（工具名以服务名为前缀）
        tools = [tool_name for tool_name in self.tool_cache.keys() if tool_name.startswith(f"{name}_")]
        
        return tools

    def _extract_description_from_schema(self, prop_info):
        """从 schema 中提取描述信息"""
        if isinstance(prop_info, dict):
            # 优先查找 description 字段
            if 'description' in prop_info:
                return prop_info['description']
            # 其次查找 title 字段
            elif 'title' in prop_info:
                return prop_info['title']
            # 检查是否有 anyOf 或 allOf 结构
            elif 'anyOf' in prop_info:
                for item in prop_info['anyOf']:
                    if isinstance(item, dict) and 'description' in item:
                        return item['description']
            elif 'allOf' in prop_info:
                for item in prop_info['allOf']:
                    if isinstance(item, dict) and 'description' in item:
                        return item['description']

        return "无描述"

    def _extract_type_from_schema(self, prop_info):
        """从 schema 中提取类型信息"""
        if isinstance(prop_info, dict):
            if 'type' in prop_info:
                return prop_info['type']
            elif 'anyOf' in prop_info:
                # 处理 Union 类型
                types = []
                for item in prop_info['anyOf']:
                    if isinstance(item, dict) and 'type' in item:
                        types.append(item['type'])
                return '|'.join(types) if types else '未知'
            elif 'allOf' in prop_info:
                # 处理 intersection 类型
                for item in prop_info['allOf']:
                    if isinstance(item, dict) and 'type' in item:
                        return item['type']

        return "未知"

    def _get_detailed_tool_info(self, tool_name: str) -> Dict[str, Any]:
        """获取工具的详细信息"""
        tool_def = self.tool_cache.get(tool_name)
        if not tool_def:
            return {}
            
        # 获取工具所属的服务
        session = self.tool_to_session_map.get(tool_name)
        service_name = None
        
        # 查找工具所属的服务
        if session:
            for name, sess in self.sessions.items():
                if sess is session:
                    service_name = name
                    break
                    
        # 提取工具信息
        if "function" in tool_def:
            function_data = tool_def["function"]
            tool_info = {
                "name": tool_name,
                "description": function_data.get("description", ""),
                "service_name": service_name,
                "inputSchema": function_data.get("parameters", {})
            }
        else:
            tool_info = {
                "name": tool_name,
                "description": tool_def.get("description", ""),
                "service_name": service_name,
                "inputSchema": tool_def.get("parameters", {})
            }
            
        return tool_info

    def get_service_details(self, name: str) -> Dict[str, Any]:
        """Get detailed information for the specified service"""
        if name not in self.sessions:
            return {}
            
        logger.info(f"Getting service details for: {name}")
        session = self.sessions.get(name)
        print(f"[DEBUG][get_service_details] name={name}, id(session)={id(session) if session else None}")
        tools = self.get_tools_for_service(name)
        last_heartbeat = self.service_health.get(name)
        
        # 获取详细工具信息
        detailed_tools = []
        for tool_name in tools:
            detailed_tool = self._get_detailed_tool_info(tool_name)
            if detailed_tool:
                detailed_tools.append(detailed_tool)
        
        return {
            "name": name,
            "tools": detailed_tools,
            "tool_count": len(tools),
            "last_heartbeat": str(last_heartbeat) if last_heartbeat else "N/A",
            "connected": name in self.sessions
        }

    def get_all_service_names(self) -> List[str]:
        """Get names of all active services"""
        return list(self.sessions.keys())

    def update_service_health(self, name: str):
        """Updates the last heartbeat time for a service."""
        if name in self.sessions:
            self.service_health[name] = datetime.now()
            logger.debug(f"Health updated for service: {name}")

    def get_last_heartbeat(self, name: str) -> Optional[datetime]:
        """Get last heartbeat time for a service"""
        return self.service_health.get(name)

    def has_service(self, name: str) -> bool:
        """Check if a service exists"""
        return name in self.sessions

    def get_service_config(self, name: str) -> Optional[Dict[str, Any]]:
        """获取服务配置"""
        if not self.has_service(name):
            return None
            
        # 从 orchestrator 的 mcp_config 获取配置
        from api.deps import app_state
        orchestrator = app_state.get("orchestrator")
        if orchestrator and orchestrator.mcp_config:
            return orchestrator.mcp_config.get_service_config(name)
            
        return None



from typing import Dict, Any
from mcpstore.core.orchestrator import MCPOrchestrator
from mcpstore.core.registry import ServiceRegistry
from fastapi import HTTPException

# 全局应用状态
app_state: Dict[str, Any] = {}

def get_orchestrator() -> MCPOrchestrator:
    orchestrator = app_state.get("orchestrator")
    if orchestrator is None:
        raise HTTPException(status_code=503, detail="Service not ready (Orchestrator not initialized)")
    return orchestrator

def get_registry() -> ServiceRegistry:
    registry = app_state.get("registry")
    if registry is None:
        raise HTTPException(status_code=503, detail="Service not ready (Registry not initialized)")
    return registry



import sys
import argparse
import uvicorn

def main():
    parser = argparse.ArgumentParser(description="mcpstore command line interface")
    subparsers = parser.add_subparsers(dest="command")

    # api 子命令
    api_parser = subparsers.add_parser("api", help="Start the mcpstore FastAPI server")
    api_parser.add_argument("--host", default="0.0.0.0", help="Host to bind")
    api_parser.add_argument("--port", type=int, default=18200, help="Port to bind")
    api_parser.add_argument("--reload", action="store_true", help="Enable auto-reload")

    args = parser.parse_args()

    if args.command == "api":
        uvicorn.run("mcpstore.scripts.app:app", host=args.host, port=args.port, reload=args.reload)
    else:
        parser.print_help()

if __name__ == "__main__":
    main() 



from fastapi import Request
from fastapi.responses import JSONResponse
from fastapi.exceptions import RequestValidationError

async def validation_exception_handler(request: Request, exc: RequestValidationError):
    errors = exc.errors()
    error_messages = []
    for error in errors:
        loc = " -> ".join([str(l) for l in error["loc"] if l != "body"])
        msg = error["msg"]
        error_messages.append(f"{loc}: {msg}")
    return JSONResponse(
        status_code=400,
        content={"detail": "Validation error", "errors": error_messages},
    ) 



import json
import logging
import os
import uuid
from datetime import datetime
from typing import List, Dict, Any, Optional

from fastapi import APIRouter, Depends, HTTPException
from pydantic import BaseModel

from api.deps import get_orchestrator, get_registry
from core.orchestrator import MCPOrchestrator
from core.registry import ServiceRegistry
from mcpstore.core.client_manager import ClientManager

router = APIRouter()
logger = logging.getLogger(__name__)

class ClientRegistrationRequest(BaseModel):
    """客户端注册请求"""
    agent_id: Optional[str] = None  # 可选的agent_id
    service_names: List[str]

class ServiceRegistrationStatus(BaseModel):
    """服务注册状态"""
    status: str  # success 或 failed
    message: Optional[str] = None
    error: Optional[str] = None

class ClientRegistrationResponse(BaseModel):
    """客户端注册响应"""
    agent_id: str  # 返回传入的或新生成的agent_id
    clients: Dict[str, Dict[str, Any]]  # client_id -> client_config
    services_status: Dict[str, ServiceRegistrationStatus]  # service_name -> status
    total_success: int
    total_failed: int
    status: str = "success"
    message: Optional[str] = None

def generate_agent_id() -> str:
    """生成agent_id"""
    timestamp = datetime.now().strftime("%Y%m%d%H%M%S")
    random_suffix = str(uuid.uuid4())[:8]
    return f"agent_{timestamp}_{random_suffix}"

def generate_client_id(agent_id: str, index: int) -> str:
    """根据agent_id生成client_id"""
    timestamp = datetime.now().strftime("%Y%m%d%H%M%S")
    return f"{agent_id}_client_{index}_{timestamp}"

def save_client_configs(configs: Dict[str, Any]):
    """保存客户端配置到文件"""
    config_dir = "configs"
    os.makedirs(config_dir, exist_ok=True)
    
    # 保存client_services.json
    client_services_path = os.path.join(config_dir, "client_services.json")
    with open(client_services_path, "w") as f:
        json.dump(configs, f, indent=2)

def save_agent_clients(agent_id: str, client_ids: List[str]):
    """保存agent和client的关系到文件"""
    config_dir = "configs"
    agent_clients_path = os.path.join(config_dir, "agent_clients.json")
    
    # 读取现有配置
    agent_clients = {}
    if os.path.exists(agent_clients_path):
        with open(agent_clients_path, "r") as f:
            agent_clients = json.load(f)
    
    # 更新配置
    agent_clients[agent_id] = client_ids
    
    # 保存配置
    with open(agent_clients_path, "w") as f:
        json.dump(agent_clients, f, indent=2)

def load_client_configs() -> Dict[str, Any]:
    """加载客户端配置"""
    config_dir = "configs"
    client_services_path = os.path.join(config_dir, "client_services.json")
    
    if os.path.exists(client_services_path):
        with open(client_services_path, "r") as f:
            return json.load(f)
    return {}

def load_agent_clients() -> Dict[str, List[str]]:
    """加载agent和client的关系"""
    config_dir = "configs"
    agent_clients_path = os.path.join(config_dir, "agent_clients.json")
    
    if os.path.exists(agent_clients_path):
        with open(agent_clients_path, "r") as f:
            return json.load(f)
    return {}

@router.post("/register_clients", response_model=ClientRegistrationResponse)
async def register_clients(
    request: ClientRegistrationRequest,
    orchestrator: MCPOrchestrator = Depends(get_orchestrator),
    registry: ServiceRegistry = Depends(get_registry)
):
    """注册客户端
    
    为agent创建多个client，每个client对应一个服务。
    如果没有提供agent_id，会自动生成一个。
    """
    try:
        # 如果没有提供agent_id，生成一个
        agent_id = request.agent_id or generate_agent_id()
        
        # 生成client配置
        client_configs = {}
        client_ids = []
        services_status = {}
        total_success = 0
        total_failed = 0
        
        for i, service_name in enumerate(request.service_names):
            try:
                # 验证服务是否存在
                service_config = registry.get_service_config(service_name)
                if not service_config:
                    raise Exception(f"Service {service_name} not found")
                
                # 生成client_id并创建配置
                client_id = generate_client_id(agent_id, i)
                client_ids.append(client_id)
                
                client_configs[client_id] = {
                    "mcpServers": {
                        service_name: service_config
                    }
                }
                
                # 记录成功状态
                services_status[service_name] = ServiceRegistrationStatus(
                    status="success",
                    message=f"Successfully registered with client {client_id}"
                )
                total_success += 1
                
            except Exception as e:
                # 记录失败状态
                services_status[service_name] = ServiceRegistrationStatus(
                    status="failed",
                    error=str(e)
                )
                total_failed += 1
                logger.error(f"Failed to register service {service_name}: {e}")
        
        if client_configs:  # 如果至少有一个服务注册成功
            # 保存配置
            save_client_configs(client_configs)
            save_agent_clients(agent_id, client_ids)
            
            # 创建会话
            session = orchestrator.session_manager.create_session(agent_id)
            
            return ClientRegistrationResponse(
                agent_id=agent_id,
                clients=client_configs,
                services_status=services_status,
                total_success=total_success,
                total_failed=total_failed,
                status="success",
                message=f"Successfully registered {total_success} clients, {total_failed} failed"
            )
        else:
            raise HTTPException(
                status_code=500,
                detail="All service registrations failed"
            )
            
    except Exception as e:
        logger.error(f"Failed to register clients: {e}")
        raise HTTPException(status_code=500, detail=str(e)) 



from fastapi import APIRouter, Depends, HTTPException, Path
from typing import Dict, Any, Optional, List
from mcpstore.core.registry import ServiceRegistry
from mcpstore.core.orchestrator import MCPOrchestrator
from mcpstore.scripts.models import (
    TransportType, 
    HTTPRegisterRequest, 
    StdioRegisterRequest, 
    ToolRegisterRequest,
    RegisterRequestUnion,
    ServiceInfo,
    ServicesResponse,
    ToolInfo,
    ToolsResponse,
    JsonRegisterRequest,
    JsonRegistrationResponse,
    ServiceRegistrationResult,
    JsonUpdateRequest,
    JsonConfigResponse
)
from mcpstore.scripts.deps import get_orchestrator, get_registry
import logging
import time
import json
import os
from mcpstore.scripts.client_registration import load_client_configs, load_agent_clients

router = APIRouter()
logger = logging.getLogger(__name__)

async def _process_registration(
    payload: RegisterRequestUnion,
    orchestrator: MCPOrchestrator,
    agent_id: Optional[str] = None
) -> Dict[str, str]:
    """统一的注册处理逻辑，使用官方方法添加服务到mcp.json并重新加载"""
    try:
        # 构建符合官方格式的服务配置
        service_name = payload.name or f"service_{int(time.time())}"
        service_config = {}
        
        # 根据不同类型的请求添加特定配置
        if isinstance(payload, HTTPRegisterRequest):
            service_config = {
                "url": str(payload.url),
                "transport": payload.transport_type,
                "headers": payload.headers or {}
            }
        elif isinstance(payload, StdioRegisterRequest):
            if payload.transport_type == TransportType.PYTHON_STDIO:
                service_config = {
                    "command": payload.command or "python",
                    "args": [payload.script_path] + (payload.args or []),
                    "env": payload.env or {},
                    "working_dir": payload.working_dir
                }
            elif payload.transport_type == TransportType.NODE_STDIO:
                service_config = {
                    "command": payload.command or "node",
                    "args": [payload.script_path] + (payload.args or []),
                    "env": payload.env or {},
                    "working_dir": payload.working_dir
                }
        elif isinstance(payload, ToolRegisterRequest):
            if payload.transport_type == TransportType.UVX_STDIO:
                service_config = {
                    "command": "uvx",
                    "args": [payload.tool_name] + (payload.args or []),
                    "env": payload.env or {}
                }
            elif payload.transport_type == TransportType.NPX_STDIO:
                service_config = {
                    "command": "npx",
                    "args": [(payload.package_name or payload.tool_name)] + (payload.args or []),
                    "env": payload.env or {}
                }
        
        # 添加服务配置到mcp.json
        # 获取当前配置
        current_config = orchestrator.mcp_config.load_config()
        if "mcpServers" not in current_config:
            current_config["mcpServers"] = {}
            
        # 添加新服务
        current_config["mcpServers"][service_name] = service_config
        
        # 保存配置
        ok = orchestrator.mcp_config.save_config(current_config)
        if not ok:
            raise Exception("Failed to save service configuration")
            
        # 重新加载所有服务
        await orchestrator.load_from_config()
        
        # 如果指定了agent_id，为该agent更新client
        if agent_id:
            if agent_id in orchestrator.agent_clients:
                # 为特定agent创建新的client
                agent_client = orchestrator.main_client
                orchestrator.agent_clients[agent_id] = agent_client
                logger.info(f"Updated client for agent {agent_id}")
            else:
                # 如果agent不存在，创建新的agent client
                agent_client = await orchestrator.register_agent_client(agent_id)
                if not agent_client:
                    logger.warning(f"Failed to register agent client: {agent_id}")
        
        # 检查服务是否健康
        is_healthy = await orchestrator.is_service_healthy(service_name)
        
        if is_healthy:
            # 获取工具数量
            tools = orchestrator.registry.get_tools_for_service(service_name)
            tool_count = len(tools)
            message = f"Service {service_name} registered successfully with {tool_count} tools"
            if agent_id:
                message += f" and connected to agent {agent_id}"
            logger.info(message)
            return {"status": "success", "message": message}
        else:
            logger.error(f"Service {service_name} registration failed: service is unhealthy")
            # 处理特定的错误情况
            orchestrator.pending_reconnection.add(service_name)
            raise HTTPException(status_code=502, detail=f"Service {service_name} is unhealthy after registration")
            
    except HTTPException as http_exc:
        raise http_exc
    except Exception as e:
        logger.error(f"Unexpected error during registration: {e}", exc_info=True)
        raise HTTPException(
            status_code=500,
            detail=f"Internal server error during registration: {str(e)}"
        )

# 统一注册接口
@router.post("/register", response_model=Dict[str, str])
async def register_service(
    payload: RegisterRequestUnion,
    agent_id: Optional[str] = None,
    orchestrator: MCPOrchestrator = Depends(get_orchestrator)
):
    """
    统一的服务注册接口，支持所有类型的服务注册
    
    - payload: 服务注册请求
    - agent_id: 可选的agent ID，指定要将服务注册到哪个agent
    """
    return await _process_registration(payload, orchestrator, agent_id)

# 删除特定类型注册接口
# @router.post("/register/http", response_model=Dict[str, str])
# async def register_http_service(
#     payload: HTTPRegisterRequest,
#     orchestrator: MCPOrchestrator = Depends(get_orchestrator)
# ):
#     """HTTP/SSE服务专用注册接口"""
#     return await _process_registration(payload, orchestrator)

# @router.post("/register/stdio", response_model=Dict[str, str])
# async def register_stdio_service(
#     payload: StdioRegisterRequest,
#     orchestrator: MCPOrchestrator = Depends(get_orchestrator)
# ):
#     """本地脚本服务专用注册接口"""
#     return await _process_registration(payload, orchestrator)

# @router.post("/register/tool", response_model=Dict[str, str])
# async def register_tool_service(
#     payload: ToolRegisterRequest,
#     orchestrator: MCPOrchestrator = Depends(get_orchestrator)
# ):
#     """工具服务专用注册接口"""
#     return await _process_registration(payload, orchestrator)

def read_mcp_json() -> Dict[str, Any]:
    """读取默认的mcp.json文件"""
    try:
        with open("@mcp.json", "r", encoding="utf-8") as f:
            return json.load(f)
    except Exception as e:
        logger.error(f"Failed to read @mcp.json: {e}")
        raise HTTPException(
            status_code=500,
            detail=f"Failed to read @mcp.json: {str(e)}"
        )

@router.post("/register/json", response_model=JsonRegistrationResponse)
async def register_json_service(
    client_id: Optional[str] = None,
    service_names: Optional[List[str]] = None,
    orchestrator: MCPOrchestrator = Depends(get_orchestrator)
):
    """
    注册服务，支持 main_client 和普通 client
    - client_id: 客户端ID，若为 main_client 则注册主客户端，否则注册普通客户端
    - service_names: 普通 client 时可指定服务名列表
    """
    try:
        if client_id == orchestrator.client_manager.main_client_id or not client_id:
            # main_client 注册，读取 mcp.json
            config = orchestrator.mcp_config.load_config()
        else:
            # 普通 client 注册，根据服务名列表生成 config
            if not service_names:
                raise HTTPException(status_code=400, detail="service_names required for non-main_client registration")
            config = orchestrator.create_client_config_from_names(service_names)
        results = await orchestrator.register_json_services(config, client_id=client_id)
        return JsonRegistrationResponse(
            client_id=results["client_id"],
            services={},  # 可扩展为详细注册结果
            total_success=results["total_services"],
            total_failed=0 if results["status"] == "success" else 1
        )
    except Exception as e:
        logger.error(f"Error during JSON registration: {e}", exc_info=True)
        raise HTTPException(
            status_code=500,
            detail=f"Internal server error during registration: {str(e)}"
        )

@router.put("/register/json", response_model=JsonRegistrationResponse)
async def update_json_service(
    payload: JsonUpdateRequest,
    orchestrator: MCPOrchestrator = Depends(get_orchestrator)
):
    """
    使用提供的配置更新服务
    
    Args:
        payload: JSON更新请求
        orchestrator: MCP编排器实例
    """
    try:
        # 调用注册方法（更新实际上是重新注册）
        results = await orchestrator.register_json_services(
            config=payload.config,
            client_id=payload.client_id
        )
        
        # 转换结果为响应模型
        return JsonRegistrationResponse(
            client_id=results["client_id"],
            services={
                name: ServiceRegistrationResult(**result) 
                if isinstance(result, dict) else result
                for name, result in results["services"].items()
            },
            total_success=results["total_success"],
            total_failed=results["total_failed"]
        )
        
    except Exception as e:
        logger.error(f"Error during JSON update: {e}", exc_info=True)
        raise HTTPException(
            status_code=500,
            detail=f"Internal server error during update: {str(e)}"
        )

@router.get("/register/json", response_model=JsonConfigResponse)
async def get_json_config(
    client_id: Optional[str] = None,
    orchestrator: MCPOrchestrator = Depends(get_orchestrator)
):
    """
    查询服务配置，支持 main_client 和普通 client
    - client_id: 客户端ID，若为 main_client 则查询主客户端配置，否则查询普通客户端配置
    """
    try:
        if not client_id or client_id == orchestrator.client_manager.main_client_id:
            config = orchestrator.mcp_config.load_config()
            return JsonConfigResponse(
                client_id=orchestrator.client_manager.main_client_id,
                config=config
            )
        else:
            config = orchestrator.client_manager.get_client_config(client_id)
            if not config:
                raise HTTPException(
                    status_code=404,
                    detail=f"Client configuration not found: {client_id}"
                )
            return JsonConfigResponse(
                client_id=client_id,
                config=config
            )
    except HTTPException:
        raise
    except Exception as e:
        logger.error(f"Error getting JSON config: {e}", exc_info=True)
        raise HTTPException(
            status_code=500,
            detail=f"Internal server error getting config: {str(e)}"
        )

@router.get("/health", response_model=Dict[str, Any])
async def get_health_status(
    registry: ServiceRegistry = Depends(get_registry),
    orchestrator: MCPOrchestrator = Depends(get_orchestrator)
):
    """获取服务健康状态"""
    services = []
    for name, config in orchestrator.mcp_config.load_config().get("mcpServers", {}).items():
        # 获取服务的真实健康状态
        is_healthy = await orchestrator.is_service_healthy(name)
        
        # 构建服务状态信息
        service_status = {
            "name": name,
            "url": config.get("url", ""),  # 如果没有 URL，则为空字符串
            "transport_type": infer_transport_type(config),
            "status": "healthy" if is_healthy else "unhealthy",
            "command": config.get("command"),  # 添加命令信息
            "args": config.get("args"),  # 添加参数列表
            "package_name": config.get("package_name")  # 添加包名信息
        }
        services.append(service_status)
        
    return {
        "orchestrator_status": "running",
        "active_services": registry.get_session_count(),
        "total_tools": registry.get_tool_count(),
        "services": services
    }

@router.get("/service_info", response_model=Dict[str, Any])
async def get_service_info(
    name: str,
    registry: ServiceRegistry = Depends(get_registry),
    orchestrator: MCPOrchestrator = Depends(get_orchestrator)
):
    """获取服务的详细信息
    
    Args:
        name: 服务名称
    """
    if not registry.has_service(name):
        raise HTTPException(status_code=404, detail=f"Service not found: {name}")
        
    # 获取基本服务信息
    service_info = registry.get_service_info(name)
    
    # 获取服务配置
    config = orchestrator.mcp_config.get_service_config(name)
    if config:
        # 添加配置信息到返回结果
        service_info.update({
            "url": config.get("url", ""),  # 如果没有 URL，则为空字符串
            "transport_type": infer_transport_type(config),
            "keep_alive": infer_keep_alive(config),
            "working_dir": config.get("working_dir"),
            "env": config.get("env"),
            "command": config.get("command"),  # 添加命令信息
            "args": config.get("args"),  # 添加参数列表
            "package_name": config.get("package_name")  # 添加包名信息
        })
    
    # 检查服务健康状态
    is_healthy = await orchestrator.is_service_healthy(name)
    service_info["status"] = "healthy" if is_healthy else "unhealthy"
    
    return service_info

def infer_transport_type(service_config):
    if not service_config:
        return TransportType.STREAMABLE_HTTP
    # 优先使用 transport 字段
    transport = service_config.get("transport")
    if transport:
        return transport
    # 其次根据 url 判断
    if service_config.get("url"):
        return TransportType.STREAMABLE_HTTP
    # 根据 command/args 判断
    cmd = (service_config.get("command") or "").lower()
    args = " ".join(service_config.get("args", [])).lower()
    if "python" in cmd or ".py" in args:
        return TransportType.PYTHON_STDIO
    if "node" in cmd or ".js" in args:
        return TransportType.NODE_STDIO
    # 其它可扩展
    return TransportType.STREAMABLE_HTTP

def infer_keep_alive(service_config):
    # 目前所有服务默认 keep_alive True，可根据需要扩展
    return True

@router.get("/services", response_model=ServicesResponse)
async def list_services(
    registry: ServiceRegistry = Depends(get_registry),
    orchestrator: MCPOrchestrator = Depends(get_orchestrator)
):
    """获取所有已注册服务的详细信息"""
    services_info = []
    total_tools = 0
    
    # 获取所有服务的详细信息
    for name, config in orchestrator.mcp_config.load_config().get("mcpServers", {}).items():
        service_tools = registry.get_tools_for_service(name)
        total_tools += len(service_tools)
        
        transport_type = infer_transport_type(config)
        keep_alive = infer_keep_alive(config)
        
        # 获取服务的真实健康状态
        is_healthy = await orchestrator.is_service_healthy(name)
        status = "healthy" if is_healthy else "unhealthy"
        
        services_info.append(ServiceInfo(
            url=config.get("url", ""),  # 如果没有 URL，则为空字符串
            name=name,
            transport_type=transport_type,
            status=status,
            tool_count=len(service_tools),
            keep_alive=keep_alive,
            working_dir=config.get("working_dir"),
            env=config.get("env"),
            last_heartbeat=registry.get_last_heartbeat(name),
            command=config.get("command"),  # 添加命令信息
            args=config.get("args"),  # 添加参数列表
            package_name=config.get("package_name")  # 添加包名信息
        ))
        
    return ServicesResponse(
        services=services_info,
        total_services=len(services_info),
        total_tools=total_tools
    )

@router.get("/tools", response_model=ToolsResponse)
async def list_tools(
    agent_id: Optional[str] = None,
    client_id: Optional[str] = None,
    registry: ServiceRegistry = Depends(get_registry),
    orchestrator: MCPOrchestrator = Depends(get_orchestrator)
):
    """获取工具列表
    
    可以通过以下方式查询：
    1. 不传任何ID - 返回所有工具
    2. 传入agent_id - 返回该agent关联的所有client的工具
    3. 传入client_id - 返回该client的工具
    """
    try:
        tools_info = []
        
        # 加载配置
        client_configs = load_client_configs()
        agent_clients = load_agent_clients()
        
        if client_id:
            # 按client_id查询
            if client_id not in client_configs:
                raise HTTPException(status_code=404, detail=f"Client {client_id} not found")
            
            client_config = client_configs[client_id]
            service_names = list(client_config["mcpServers"].keys())
            
            # 获取这些服务的工具
            for service_name in service_names:
                tool_names = registry.get_tools_for_service(service_name)
                for tool_name in tool_names:
                    tool_info = registry._get_detailed_tool_info(tool_name)
                    if tool_info:
                        tools_info.append(ToolInfo(
                            name=tool_info["name"],
                            description=tool_info["description"],
                            service_name=service_name,
                            client_id=client_id,
                            inputSchema=tool_info.get("inputSchema")
                        ))
                    
        elif agent_id:
            # 按agent_id查询
            if agent_id not in agent_clients:
                raise HTTPException(status_code=404, detail=f"Agent {agent_id} not found")
                
            client_ids = agent_clients[agent_id]
            
            # 获取所有关联client的工具
            for client_id in client_ids:
                if client_id in client_configs:
                    client_config = client_configs[client_id]
                    service_names = list(client_config["mcpServers"].keys())
                    
                    for service_name in service_names:
                        tool_names = registry.get_tools_for_service(service_name)
                        for tool_name in tool_names:
                            tool_info = registry._get_detailed_tool_info(tool_name)
                            if tool_info:
                                tools_info.append(ToolInfo(
                                    name=tool_info["name"],
                                    description=tool_info["description"],
                                    service_name=service_name,
                                    client_id=client_id,
                                    inputSchema=tool_info.get("inputSchema")
                                ))
                            
        else:
            # 返回所有工具
            for tool in registry.get_all_tool_info():
                tools_info.append(ToolInfo(
                    name=tool["name"],
                    description=tool["description"],
                    service_name=tool.get("service_name"),
                    client_id=None,  # 全局工具没有关联的client_id
                    inputSchema=tool.get("inputSchema")
                ))
        
        return ToolsResponse(
            tools=tools_info,
            total_tools=len(tools_info)
        )
        
    except Exception as e:
        logger.error(f"Error listing tools: {e}")
        raise HTTPException(status_code=500, detail=str(e))




import os, sys
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

"""
MCP服务编排器

该模块提供了MCPOrchestrator类，用于管理MCP服务的连接、工具调用和查询处理。
它是FastAPI应用程序的核心组件，负责协调客户端和服务之间的交互。
"""

import asyncio
import logging
from typing import Dict, List, Any, Optional, Tuple, Set, Union, AsyncGenerator
from datetime import datetime, timedelta
from urllib.parse import urljoin

from mcpstore.core.registry import ServiceRegistry
from mcpstore.core.client_manager import ClientManager
from fastmcp import Client
from fastmcp.client.transports import (
    MCPConfigTransport,
    StreamableHttpTransport,
    SSETransport,
    PythonStdioTransport,
    NodeStdioTransport,
    UvxStdioTransport,
    NpxStdioTransport
)
from mcpstore.plugins.json_mcp import MCPConfig
from mcpstore.scripts.models import TransportType, ServiceRegistrationResult
from mcpstore.core.session_manager import SessionManager

logger = logging.getLogger(__name__)

class MCPOrchestrator:
    """
    MCP服务编排器
    
    负责管理服务连接、工具调用和查询处理。
    """
    
    def __init__(self, config: Dict[str, Any], registry: ServiceRegistry):
        """
        初始化MCP编排器
        
        Args:
            config: 配置字典
            registry: 服务注册表实例
        """
        self.config = config
        self.registry = registry
        self.clients: Dict[str, Client] = {}  # key为mcpServers的服务名
        self.main_client: Optional[Client] = None
        self.main_client_ctx = None  # async context manager for main_client
        self.main_config = {"mcpServers": {}}  # 中央配置
        self.agent_clients: Dict[str, Client] = {}  # agent_id -> client映射
        self.pending_reconnection: Set[str] = set()
        self.react_agent = None
        
        # 从配置中获取心跳和重连设置
        timing_config = config.get("timing", {})
        self.heartbeat_interval = timedelta(seconds=int(timing_config.get("heartbeat_interval_seconds", 60)))
        self.heartbeat_timeout = timedelta(seconds=int(timing_config.get("heartbeat_timeout_seconds", 180)))
        self.reconnection_interval = timedelta(seconds=int(timing_config.get("reconnection_interval_seconds", 60)))
        self.http_timeout = int(timing_config.get("http_timeout_seconds", 10))
        
        # 监控任务
        self.heartbeat_task = None
        self.reconnection_task = None
        self.mcp_config = MCPConfig()
        
        # 客户端管理器
        self.client_manager = ClientManager()
        
        # 会话管理器
        self.session_manager = SessionManager()
    
    async def setup(self):
        """初始化编排器资源（不再做服务注册）"""
        logger.info("Setting up MCP Orchestrator...")
        # 只做必要的资源初始化
        pass
    
    async def start_monitoring(self):
        """启动后台健康检查和重连监视器"""
        logger.info("Starting monitoring tasks...")
        
        # 启动心跳监视器
        if self.heartbeat_task is None or self.heartbeat_task.done():
            logger.info(f"Starting heartbeat monitor. Interval: {self.heartbeat_interval.total_seconds()}s")
            self.heartbeat_task = asyncio.create_task(self._heartbeat_loop())
        
        # 启动重连监视器
        if self.reconnection_task is None or self.reconnection_task.done():
            logger.info(f"Starting reconnection monitor. Interval: {self.reconnection_interval.total_seconds()}s")
            self.reconnection_task = asyncio.create_task(self._reconnection_loop())
    
    async def _heartbeat_loop(self):
        """后台循环，用于定期健康检查"""
        while True:
            await asyncio.sleep(self.heartbeat_interval.total_seconds())
            await self._check_services_health()
    
    async def _check_services_health(self):
        """检查所有服务的健康状态"""
        logger.debug("Running periodic health check for all services...")
        for name in self.clients:
            try:
                is_healthy = await self.is_service_healthy(name)
                if is_healthy:
                    logger.debug(f"Health check SUCCESS for: {name}")
                    self.registry.update_service_health(name)
                else:
                    logger.warning(f"Health check FAILED for {name}")
                    self.pending_reconnection.add(name)
            except Exception as e:
                logger.warning(f"Health check error for {name}: {e}")
                self.pending_reconnection.add(name)
    
    async def _reconnection_loop(self):
        """定期尝试重新连接服务的后台循环"""
        while True:
            await asyncio.sleep(self.reconnection_interval.total_seconds())
            await self._attempt_reconnections()
    
    async def _attempt_reconnections(self):
        """尝试重新连接所有待重连的服务"""
        if not self.pending_reconnection:
            return  # 如果没有待重连的服务，跳过
        
        # 创建副本以避免迭代过程中修改集合的问题
        names_to_retry = list(self.pending_reconnection)
        logger.info(f"Attempting to reconnect {len(names_to_retry)} service(s): {names_to_retry}")
        
        for name in names_to_retry:
            try:
                # 尝试重新连接
                success, message = await self.connect_service(name)
                if success:
                    logger.info(f"Reconnection successful for: {name}")
                    self.pending_reconnection.discard(name)
                else:
                    logger.warning(f"Reconnection attempt failed for {name}: {message}")
                    # 保持name在pending_reconnection中，等待下一个周期
            except Exception as e:
                logger.warning(f"Reconnection attempt failed for {name}: {e}")
    
    async def connect_service(self, name: str, url: str = None) -> Tuple[bool, str]:
        """
        连接到指定的服务
        
        Args:
            name: 服务名称
            url: 服务URL（可选，如果不提供则从配置中获取）
            
        Returns:
            Tuple[bool, str]: (是否成功, 消息)
        """
        try:
            # 获取服务配置
            service_config = self.mcp_config.get_service_config(name)
            if not service_config:
                return False, f"Service configuration not found for {name}"
                
            # 如果提供了URL，更新配置
            if url:
                service_config["url"] = url
                
            # 创建新的客户端
            client = Client({"mcpServers": {name: service_config}})
            
            # 尝试连接
            try:
                await client.list_tools()
                self.clients[name] = client
                logger.info(f"Service {name} connected successfully")
                return True, "Connected successfully"
            except Exception as e:
                logger.error(f"Failed to connect to service {name}: {e}")
                return False, str(e)
                
        except Exception as e:
            logger.error(f"Failed to connect service {name}: {e}")
            return False, str(e)
    
    async def disconnect_service(self, url_or_name: str) -> bool:
        """从配置中移除服务并更新main_client"""
        logger.info(f"Removing service: {url_or_name}")
        
        # 查找要移除的服务名
        name_to_remove = None
        for name, server in self.main_config.get("mcpServers", {}).items():
            if name == url_or_name or server.get("url") == url_or_name:
                name_to_remove = name
                break
                
        if name_to_remove:
            # 从main_config中移除
            if name_to_remove in self.main_config["mcpServers"]:
                del self.main_config["mcpServers"][name_to_remove]
                
            # 从配置文件中移除
            ok = self.mcp_config.remove_service(name_to_remove)
            if not ok:
                logger.warning(f"Failed to remove service {name_to_remove} from configuration file")
                
            # 从registry中移除
            self.registry.remove_service(name_to_remove)
                
            # 重新创建main_client
            if self.main_config.get("mcpServers"):
                self.main_client = Client(self.main_config)
                
                # 更新所有agent_clients
                for agent_id in list(self.agent_clients.keys()):
                    self.agent_clients[agent_id] = Client(self.main_config)
                    logger.info(f"Updated client for agent {agent_id} after removing service")
                
            else:
                # 如果没有服务了，清除main_client
                self.main_client = None
                # 清除所有agent_clients
                self.agent_clients.clear()
                
            return True
        else:
            logger.warning(f"Service {url_or_name} not found in configuration.")
            return False
    
    async def refresh_services(self):
        """手动刷新所有服务连接（重新加载mcp.json）"""
        await self.load_from_config()
    
    async def is_service_healthy(self, name: str) -> bool:
        """
        检查服务是否健康
        
        Args:
            name: 服务名
            
        Returns:
            bool: 服务是否健康
        """
        try:
            # 获取服务配置
            service_config = self.mcp_config.get_service_config(name)
            if not service_config:
                logger.warning(f"Service configuration not found for {name}")
                return False
            
            # 创建新的客户端并在上下文管理器中使用 ping 方法
            client = Client({"mcpServers": {name: service_config}})
            try:
                async with client as session:
                    await session.ping()
                    return True
            except Exception as e:
                logger.warning(f"Health check failed for {name}: {e}")
                return False
        except Exception as e:
            logger.warning(f"Health check failed for {name}: {e}")
            return False
    
    # async def process_unified_query(
    #     self,
    #     query: str,
    #     agent_id: Optional[str] = None,
    #     mode: str = "react",
    #     include_trace: bool = False
    # ) -> Union[str, Dict[str, Any]]:
    #     """处理统一查询"""
    #     # 获取或创建会话
    #     session = self.session_manager.get_or_create_session(agent_id)
    #
    #     if not session.tools:
    #         # 如果会话没有工具，加载所有可用工具
    #         for service_name, client in self.clients.items():
    #             try:
    #                 tools = await client.list_tools()
    #                 for tool in tools:
    #                     session.add_tool(tool.name, {
    #                         "name": tool.name,
    #                         "description": tool.description,
    #                         "inputSchema": tool.inputSchema if hasattr(tool, "inputSchema") else None
    #                     }, service_name)
    #                     session.add_service(service_name, client)
    #             except Exception as e:
    #                 logger.error(f"Failed to load tools from service {service_name}: {e}")
    #
    #     # 处理查询...
    #     return {"result": "query processed", "session_id": session.agent_id}
    
    async def execute_tool(
        self,
        service_name: str,
        tool_name: str,
        parameters: Dict[str, Any],
        agent_id: Optional[str] = None
    ) -> Any:
        """执行工具"""
        if agent_id:
            # 如果提供了agent_id，从会话中获取client
            session = self.session_manager.get_session(agent_id)
            if session:
                client = session.services.get(service_name)
                if not client:
                    raise Exception(f"Service {service_name} not found in session")
            else:
                raise Exception(f"Session not found for agent {agent_id}")
        else:
            # 否则使用默认client
            client = self.clients.get(service_name)
            if not client:
                raise Exception(f"Service {service_name} not found")
        
        try:
            async with client:
                result = await client.call_tool(tool_name, parameters)
                logger.info(f"Tool {tool_name} executed successfully")
                return result
        except Exception as e:
            logger.error(f"Failed to execute tool {tool_name}: {e}")
            raise Exception(f"Tool execution failed: {str(e)}")
    
    async def cleanup(self):
        """清理资源"""
        logger.info("Cleaning up MCP Orchestrator resources...")
        
        # 清理会话
        self.session_manager.cleanup_expired_sessions()
        
        # 停止监控任务
        if self.heartbeat_task and not self.heartbeat_task.done():
            self.heartbeat_task.cancel()
            try:
                await self.heartbeat_task
            except asyncio.CancelledError:
                pass
                
        if self.reconnection_task and not self.reconnection_task.done():
            self.reconnection_task.cancel()
            try:
                await self.reconnection_task
            except asyncio.CancelledError:
                pass
                
        # 关闭所有客户端连接
        for name, client in self.clients.items():
            try:
                await client.close()
            except Exception as e:
                logger.error(f"Error closing client {name}: {e}")
                
        self.clients.clear()
        self.pending_reconnection.clear()
        
    async def register_agent_client(self, agent_id: str, config: Optional[Dict[str, Any]] = None) -> Client:
        """
        为agent注册一个新的client实例
        
        Args:
            agent_id: 代理ID
            config: 可选的配置，如果为None则使用main_config
            
        Returns:
            新创建的Client实例
        """
        # 使用main_config或提供的config创建新的client
        agent_config = config or self.main_config
        agent_client = Client(agent_config)
        
        # 存储agent_client
        self.agent_clients[agent_id] = agent_client
        logger.info(f"Registered agent client for {agent_id}")
        
        return agent_client

    def get_agent_client(self, agent_id: str) -> Optional[Client]:
        """
        获取agent的client实例
        
        Args:
            agent_id: 代理ID
            
        Returns:
            Client实例或None
        """
        return self.agent_clients.get(agent_id)

    async def start_main_client(self, config: Dict[str, Any]):
        """启动 main_client 的 async with 生命周期，注册服务和工具"""
        self.main_client = Client(config)
        self.main_client_ctx = self.main_client.__aenter__()
        try:
            await self.main_client_ctx
            logger.info(f"main_client connected: {self.main_client.is_connected()}")
            logger.info(f"main_client transport: {self.main_client.transport}")
            logger.info(f"main_client session: {getattr(self.main_client, 'session', None)}")
            try:
                tool_list = await self.main_client.list_tools()
                all_tools = []
                for tool in tool_list:
                    # 正确处理FastMCP的Tool对象
                    tool_name = tool.name
                    tool_def = {
                        "type": "function",
                        "function": {
                            "name": tool_name,
                            "description": tool.description,
                            "parameters": {
                                "type": "object",
                                "properties": {},
                                "required": []
                            }
                        }
                    }
                    
                    # 处理参数信息
                    if hasattr(tool, 'inputSchema') and tool.inputSchema:
                        schema = tool.inputSchema
                        if isinstance(schema, dict):
                            tool_def["function"]["parameters"] = schema
                        
                    all_tools.append((tool_name, tool_def))
                    
                logger.info(f"Retrieved {len(all_tools)} tools from main_client")
            except Exception as e:
                logger.warning(f"Failed to get tools from main_client: {e}")
                all_tools = []
            
            self.clients.clear()
            self.registry.clear()
            for name, service_config in config.get("mcpServers", {}).items():
                try:
                    self.clients[name] = self.main_client
                    self.registry.add_service(name, self.main_client, all_tools, name)
                except Exception as e:
                    logger.error(f"Failed to register service {name}: {e}")
        except Exception as e:
            logger.error(f"main_client MCP服务连接失败: {e}")
            raise

    async def stop_main_client(self):
        """优雅关闭 main_client 的 async with 生命周期"""
        if self.main_client and self.main_client_ctx:
            try:
                await self.main_client.__aexit__(None, None, None)
                logger.info("main_client closed.")
            except Exception as e:
                logger.error(f"Error closing main_client: {e}")
        self.main_client = None
        self.main_client_ctx = None

    async def agent_client_context(self, config: Dict[str, Any]):
        """agent client 的 async with 临时上下文管理器"""
        client = Client(config)
        try:
            async with client:
                logger.info(f"agent_client connected: {client.is_connected()}")
                logger.info(f"agent_client transport: {client.transport}")
                logger.info(f"agent_client session: {getattr(client, 'session', None)}")
                yield client
        except Exception as e:
            logger.error(f"agent_client 连接或操作失败: {e}")
            raise

    async def register_json_services(self, config: Dict[str, Any], client_id: Optional[str] = None) -> Dict[str, Any]:
        """注册JSON配置中的服务"""
        try:
            # 创建客户端
            client = Client(config)
            
            # 获取工具列表
            try:
                # 获取所有服务名称
                service_names = list(config.get("mcpServers", {}).keys())
                if not service_names:
                    logger.warning("No services found in configuration")
                    return {
                        "client_id": client_id or "main_client",
                        "services": {},
                        "total_success": 0,
                        "total_failed": 0
                    }
                
                async with client:    
                    # 获取工具列表
                    tool_list = await client.list_tools()
                    if not tool_list:
                        logger.warning("No tools found")
                        return {
                            "client_id": client_id or "main_client",
                            "services": {},
                            "total_success": 0,
                            "total_failed": 0
                        }
                        
                    # 处理工具列表
                    all_tools = []
                    for tool in tool_list:
                        # 正确处理FastMCP的Tool对象
                        tool_name = tool.name
                        
                        # 根据工具名前缀确定所属服务
                        service_name = None
                        for name in service_names:
                            if tool_name.startswith(f"{name}_"):
                                service_name = name
                                break
                                
                        if not service_name:
                            logger.warning(f"Tool {tool_name} does not belong to any service, skipping")
                            continue
                            
                        # 处理参数信息
                        parameters = {}
                        if hasattr(tool, 'inputSchema') and tool.inputSchema:
                            parameters = tool.inputSchema
                        elif hasattr(tool, 'parameters') and tool.parameters:
                            parameters = tool.parameters
                            
                        tool_def = {
                            "type": "function",
                            "function": {
                                "name": tool_name,
                                "description": tool.description,
                                "parameters": parameters
                            }
                        }
                        all_tools.append((tool_name, tool_def))
                    
                    # 为每个服务注册其工具
                    for service_name in service_names:
                        # 过滤出属于当前服务的工具
                        service_tools = [(name, tool_def) for name, tool_def in all_tools if name.startswith(f"{service_name}_")]
                        logger.info(f"Filtered {len(service_tools)} tools for service {service_name}")
                        
                        # 注册服务及其工具
                        self.registry.add_service(service_name, client, service_tools)
                        
                        # 保存客户端实例
                        self.clients[service_name] = client
                        
                    return {
                        "client_id": client_id or "main_client",
                        "services": {
                            name: {"status": "success", "message": "Service registered successfully"}
                            for name in service_names
                        },
                        "total_success": len(service_names),
                        "total_failed": 0
                    }
            except Exception as e:
                logger.error(f"Error retrieving tools: {e}", exc_info=True)
                return {
                    "client_id": client_id or "main_client",
                    "services": {},
                    "total_success": 0,
                    "total_failed": 1,
                    "error": str(e)
                }
        except Exception as e:
            logger.error(f"Error registering services: {e}", exc_info=True)
            return {
                "client_id": client_id or "main_client",
                "services": {},
                "total_success": 0,
                "total_failed": 1,
                "error": str(e)
            }

    def create_client_config_from_names(self, service_names: list) -> Dict[str, Any]:
        """
        根据服务名列表，从 mcp.json 生成新的 client config
        """
        all_services = self.mcp_config.load_config().get("mcpServers", {})
        selected = {name: all_services[name] for name in service_names if name in all_services}
        return {"mcpServers": selected} 



import logging
import os
import sys
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
from fastapi import FastAPI
from fastapi.middleware.cors import CORSMiddleware
from api.exception_handlers import validation_exception_handler

from mcpstore.scripts.service_management import router as service_router
from api.tool_execution import router as tool_execution_router
from api.client_registration import router as client_registration_router
from api.deps import app_state
from plugins.json_mcp import MCPConfig
from core.registry import ServiceRegistry
from core.orchestrator import MCPOrchestrator

logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(name)s - %(levelname)s - %(message)s',
                    handlers=[logging.StreamHandler(sys.stdout), logging.FileHandler("mcp_service.log")])
logger = logging.getLogger(__name__)

async def lifespan(app: FastAPI):
    logger.info("Application startup: Initializing components...")
    config_path = os.path.join(os.path.dirname(os.path.dirname(__file__)), "plugins", "mcp.json")
    mcp_config_handler = MCPConfig(config_path)
    config = mcp_config_handler.load_config()
    registry = ServiceRegistry()
    orchestrator = MCPOrchestrator(config=config, registry=registry)
    await orchestrator.setup()
    await orchestrator.start_monitoring()
    logger.info("Registering main_client from mcp.json via unified logic...")
    await orchestrator.register_json_services(config, client_id="main_client")
    app_state["orchestrator"] = orchestrator
    app_state["registry"] = registry
    app_state["mcp_config"] = mcp_config_handler
    logger.info("Components initialized and background tasks started.")
    try:
        yield
    finally:
        logger.info("Application shutdown: Cleaning up resources...")
        orch = app_state.get("orchestrator")
        if orch:
            await orch.stop_main_client()
            await orch.cleanup()
        app_state.clear()
        logger.info("Application shutdown complete.")

app = FastAPI(lifespan=lifespan)

# Add CORS middleware
app.add_middleware(
    CORSMiddleware,
    allow_origins=["*"],
    allow_credentials=True,
    allow_methods=["*"],
    allow_headers=["*"],
)

# 注册API路由
# app.include_router(llm_agent_router)
app.include_router(service_router)
app.include_router(tool_execution_router)
app.include_router(client_registration_router)

# 注册异常处理
from fastapi.exceptions import RequestValidationError
app.add_exception_handler(RequestValidationError, validation_exception_handler) 
